//======================================================================
//-----------------------------------------------------------------------
/**
 * @file		iutest_core.hpp
 * @brief		iris unit test core t@C
 *
 * @author		t.sirayanagi
 * @version		1.0
 *
 * @par			copyright
 * Copyright (C) 2011-2012, Takazumi Shirayanagi\n
 * The new BSD License is applied to this software.
 * see LICENSE
*/
//-----------------------------------------------------------------------
//======================================================================
#ifndef INCG_IRIS_iutest_core_HPP_33560431_783B_4ae7_8FC8_6E78355CAF87_
#define INCG_IRIS_iutest_core_HPP_33560431_783B_4ae7_8FC8_6E78355CAF87_

//======================================================================
// include
#include "internal/iutest_core_impl.hpp"
#include "internal/iutest_params_util.hpp"
#include "internal/iutest_file.h"

namespace iutest
{

//======================================================================
// class
/**
 * @brief	eXgŜ̊Ǘ
*/
class UnitTest : public UnitTestImpl
{
public:
	/** @private */
	static	UnitTest&	instance(void)		{ static UnitTest inst; return inst; }
	/** 
	 * @brief	UnitTest CX^X̎擾
	*/
	static	UnitTest*	GetInstance(void)	{ return &instance(); }

public:
	/**
	 * @brief	eXg TestCase ̎擾
	*/
	const TestCase*		current_test_case(void)		const	{ return m_current_testcase; }

	/**
	 * @brief	eXg TestInfo ̎擾
	 * @note	݊̂ o֐ ɂĂ܂B
	*/
	const TestInfo*		current_test_info(void)		const	{ return Test::GetCurrentTestInfo(); }

	/**
	 * @brief	eXg̃eXg TestResult ̎擾
	*/
	const TestResult*	current_test_result(void)	const	{ return current_test_info() ? current_test_info()->result() : NULL; }

	/** V[h̎擾 */
	int				random_seed(void)			const	{ return TestEnv::get_random_seed(); }

public:
	/** eXg */
	int				total_test_count(void)		const	{ return m_total_test_num; }
	/** seXg */
	int				test_to_run_count(void)		const	{ return m_should_run_num; }
	/** seXg */
	int				failed_test_count(void)		const	{ return m_failure_num; }
	/** eXg */
	int				disabled_test_count(void)	const	{ return m_disable_num; }
	/** eXg */
	int				successful_test_count(void)	const	{ return test_to_run_count() - failed_test_count(); }
	/** XLbveXg */
	int				skip_test_count(void)		const	{ return total_test_count() - test_to_run_count(); }

	/** eXgP[X̑ */
	int				total_test_case_count(void)	const	{ return m_testcases.count(); }
	/** seXgP[X */
	int				test_case_to_run_count(void)	const
	{
		int count=0;
		for( iuTestCases::iterator it = m_testcases.begin(), end=m_testcases.end(); it != end; ++it )
		{
			if( !it->should_run() ) continue;
			++count;
		}
		return count;
	}
	/** eXgP[X */
	int				successful_test_case_count(void)	const
	{
		int count=0;
		for( iuTestCases::iterator it = m_testcases.begin(), end=m_testcases.end(); it != end; ++it )
		{
			if( !it->should_run() ) continue;
			if( it->failed_test_count() == 0 ) ++count;
		}
		return count;
	}
	/** seXgP[X */
	int				failed_test_case_count(void)	const
	{
		int count=0;
		for( iuTestCases::iterator it = m_testcases.begin(), end=m_testcases.end(); it != end; ++it )
		{
			if( !it->should_run() ) continue;
			if( it->failed_test_count() > 0 ) ++count;
		}
		return count;
	}

	/** eXg̎s~b */
	TimeInMillisec	elapsed_time(void)		const	{ return m_elapsedmsec; }

	/** eXgP[X̎擾 */
	const TestCase*	GetTestCase(int index)	const	{ return m_testcases[index]; }

	/** eXgǂ */
	bool			Passed(void)			const	{ return m_failure_num == 0; }
	/** eXgsǂ */
	bool			Failed(void)			const	{ return m_failure_num != 0; }

	/** CxgXi[̎擾 */
	TestEventListeners&	listeners(void)		const	{ return TestEnv::event_listeners(); }

protected:
	/**
	 * @brief	eXg̎s
	*/
	int Run(void)
	{
		if( !PreRunner() ) return 0;

		int ret = 1;
#if IUTEST_HAS_EXCEPTIONS
		if( TestFlag::IsEnableFlag(TestFlag::CATCH_EXCEPTION_GLOBAL) )
		{
			try
			{
#if IUTEST_HAS_SEH
				ret = RunOnMSC();
#else
				ret = RunImpl();
#endif
			}
			catch( ... )
			{
			}
		}
		else
#endif
		{
			ret = RunImpl();
		}
		return ret;
	}
protected:
#if IUTEST_HAS_EXCEPTIONS && IUTEST_HAS_SEH
	int	RunOnMSC(void)
	{
		_EXCEPTION_POINTERS* info = NULL;
		int ret = 1;
		__try
		{
			ret = RunImpl();
		}
		__except (info = GetExceptionInformation()
			, detail::seh_exception::should_process_through_break_and_cppexceptions(GetExceptionCode()))
		{
			detail::seh_exception::translator(GetExceptionCode(), info);
		}
		return ret;
	}
#endif

private:
	int RunImpl(void)
	{
		TestProgramStart();

		int repeat = TestEnv::get_repeat_count();

		int count = repeat > 1 ? 1 : 0;
		bool result = true;
		while(repeat)
		{
			listeners().OnTestIterationStart(*this, count);
			if( !RunOnce() )
			{
				result = false;
			}
			listeners().OnTestIterationEnd(*this, count);

			++count;
			if( repeat > 0 ) --repeat;
		}

		TestProgramEnd();

		return result ? 0 : 1;
	}

	/**
	 * @brief	vO̊Jn
	*/
	void	TestProgramStart(void)
	{
#if 0
#if IUTEST_HAS_EXCEPTIONS && defined(_MSC_VER) && IUTEST_HAS_SEH
		// /EHa IvVKv
		if( TestFlag::IsEnableFlag(TestFlag::CATCH_EXCEPTION) )
		{
			// SEH OɑΉ
			_set_se_translator(detail::seh_exception::translator);
		}
#endif
#endif

		// tB^O
		m_should_run_num = 0;
		m_disable_num = 0;
		for( iuTestCases::iterator it = m_testcases.begin(), end=m_testcases.end(); it != end; ++it )
		{
			it->filter();
			m_should_run_num += it->test_to_run_count();
			m_disable_num += it->disabled_test_count();
		}

		atexit(OnExit);

		listeners().OnTestProgramStart(*this);
	}

	/**
	 * @brief	ZbgAbv
	*/
	void	EnvironmentSetUp(void)
	{
		listeners().OnEnvironmentsSetUpStart(*this);
		for( iuEnvironmentList::iterator it = TestEnv::environments().begin(), end=TestEnv::environments().end(); it != end; ++it )
		{
			(*it)->SetUp();
		}
		listeners().OnEnvironmentsSetUpEnd(*this);
	}

	/**
	 * @brief	eXg̎siP[vj
	*/
	bool	RunOnce(void)
	{
		bool result = true;
		m_failure_num = 0;
		m_elapsedmsec = 0;

		// eXgʂ̃NA
		for( iuTestCases::iterator it=m_testcases.begin(), end=m_testcases.end(); it != end; ++it )
		{
			it->clear();
		}

		TestEnv::SetUp();

		// Vbt
		if( TestFlag::IsEnableFlag(TestFlag::SHUFFLE_TESTS) )
		{
			m_testcases.shuffle(TestEnv::genrand());
		}

		// O[oZbg
		EnvironmentSetUp();

		for( iuTestCases::iterator it=m_testcases.begin(), end=m_testcases.end(); it != end; ++it )
		{
			if( !it->should_run() ) continue;

			m_current_testcase = *it;
			if( !it->Run() )
			{
				result = false;
			}
			m_current_testcase = NULL;
			int failed = it->failed_test_count();
			m_elapsedmsec += it->elapsed_time();
			m_failure_num += failed;
		}

		// O[oZbg
		EnvironmentTearDown();

		return result;
	}

	/**
	 * @brief	
	*/
	void	EnvironmentTearDown(void)
	{
		listeners().OnEnvironmentsTearDownStart(*this);
		for( iuEnvironmentList::iterator it = TestEnv::environments().begin(), end=TestEnv::environments().end(); it != end; ++it )
		{
			(*it)->TearDown();
		}
		listeners().OnEnvironmentsTearDownEnd(*this);
	}

	/**
	 * @brief	vȌI
	*/
	void	TestProgramEnd(void)
	{
		static bool pass = false;
		if( pass ) return;
		listeners().OnTestProgramEnd(*this);
		pass = true;
	}

private:
	// C̏IR[obN֐
	static void	OnExit(void)
	{
		instance().TestProgramEnd();
	}

private:
	UnitTest(void)
		: m_current_testcase(NULL)
	{
		// ϐɂeXgtOw[h
		TestEnv::LoadEnviromentVariable();
	}

private:
	// 
	void	Initialize(void)
	{
#if IUTEST_HAS_PARAM_TEST
		m_param_testcase_holder.RegisterTests();
#endif

		// t@CVXȅ
		if( detail::IFileSystem::GetInstance() == NULL )
		{
#if defined(IUTEST_FILE)
			static FileSystem<IUTEST_FILE> filesystem;
			filesystem.Initialize();
#elif IUTEST_HAS_FOPEN
			static FileSystem<StdioFile> filesystem;
			filesystem.Initialize();
#endif
		}
	}

#if IUTEST_HAS_PARAM_TEST
public:
	/** @private */
	ParamTestCaseHolder&	parameterized_test_registry(void)	{ return m_param_testcase_holder; }
private:
	ParamTestCaseHolder	m_param_testcase_holder;
#endif

private:
	friend class UnitTestSource;

	const TestCase*	m_current_testcase;
};

namespace detail
{

/**
 * @private
 * @internal
 * @brief	P̃eXgCX^X
*/
template<class Tester>
class TestInstance
{
public:
	// RXgN^
	TestInstance(const char* testcase, const char* name, TestTypeId id, SetUpMethod setup, TearDownMethod teardown)
		: m_mediator(UnitTest::instance().AddTestCase<TestCase>(testcase, id, setup, teardown))
		, m_info(&m_mediator, name, &m_factory)
	{
		UnitTest::instance().AddTestInfo(m_mediator.ptr(), &m_info);
	}

private:
	TestCaseMediator	m_mediator;
	TestInfo			m_info;
	iuFactory<Tester>	m_factory;
};

}	// end of namespace detail

}	// end of namespace iutest

#endif
